/* Animation.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef ANIMATION_H_
#define ANIMATION_H_
#include <list>
#include <iostream>
#include <cmath>
#include "Transform.h"
#include "Vector.h"
using std::list;
using std::cout;

/**
 * Klasse für zyklische Animationen. Jede Animation registriert sich in der als
 * Klassenvariablen geführten Liste und sobald die Methode nextTick aufgerufen wird,
 * werden alle Animations-Instanzen in der Liste einen Tick weiter bewegt.
 * Eine zyklische Animation hat eine Wellenform, eine Periode und eine Amplitude und kann
 * auf einer Kombination von Feldern einer Transformation wirken.
 */
class Animation
{
private:
	/**
	 * Die Transformation, deren Werte in der Animation bearbeitet werden
	 */
	Transform* target;
	
	/**
	 * Der Typ (Wellenform) der Animation. Muss einer der statischen Konstanten ANI_* sein
	 */
	int aniType;
	
	/**
	 * Die Periode (Wellenbreite) der Animation. Ist proportional zur Aufruffrequenz
	 * der nextTick()-Methode.
	 */
	double period;
	
	/**
	 * Die Amplitude (Wellenhöhe) der Animation. Gibt daher z.B. die maximale Auslenkung
	 * bei Bewegungsanimationen an
	 */
	double amplitude;
	
	/**
	 * Die Auslenkung (vertikale Wellenverschiebung). Ist Auslenkung = Amplitude, so liegt
	 * der Wertebereich der Welle zwischen 0 und der Amplitude.
	 */
	double displacement;
	
	/**
	 * Feldmaske: Stellt ein, welche Komponenten des Transformationsvektors beeinflusst werden
	 * 1000 - x-Komponente des Vektors beeinflusst
	 * 0100 - y-Komponente des Vektors beeinflusst
	 * 0010 - z-Komponente des Vektors beeinflusst
	 * 0001 - w-Komponente des Vektors beeinflusst
	 * 0x6 ändert also z.B. simultan y- und z- Komponente
	 */
	int fieldMask;
	
	/**
	 * Klassenvariable: Liste der Animationen, die bei nextTick() weiterbewegt werden sollen
	 */
	static list<Animation*> animations;
	
	/**
	 * Methode, die eine neu erzeuge Animation aufruft, um sich zu registrieren
	 */
	static void registerNew(Animation* ani);
	
public:
	/**
	 * Eine Animation soll entfernt werden
	 */
	static void remove(Transform* t);
	
	/**
	 * Wellenform: Sinusförmig. Berechnungsvorschrift: (sin(tick/periode)*amplitude)+displacement
	 */
	static const int ANI_SINUSOIDAL = 0x0001;
	
	/**
	 * Wellenform: Sägezahn. Berechnungsvorschrift: (tick mod amplitude)*periode
	 */
	static const int ANI_SAWTOOTH = 0x0002;
	
	/**
	 * Konstruktor zum Anlegen einer neuen Animation. Das reine Erzeugen des Objektes reicht,
	 * um die Animation zu starten.
	 */
	Animation(Transform* target, int aniType, double period, double amplitude,
			double displacement, int fieldMask);
	
	/**
	 * Alle in der statischen Liste registrierten Animationen sollen einen Tick weiterbewegt werden
	 */
	static void nextTick(unsigned long tick);
	
	/**
	 * Default-Destruktor
	 */
	virtual ~Animation();
};

#endif /*ANIMATION_H_*/
