/* Fish.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FISH_H_
#define FISH_H_
#include <list>
#include "Object.h"
#include "Children.h"
#include "Animation.h"
#include "Translation.h"
#include "Rotation.h"
#include "Polygon.h"
#include "Sphere.h"
#include "Vector.h"
#include "config.h"
using std::list;

/**
 * Die Fisch-Klasse bringt eine statische Liste mit, die für die Bewegung als Schwarm
 * verwendet wird. Jedes einzelne Fisch-Objekt agiert als Objekt-Knoten (draw()-Methode).
 */
class Fish : public Object {
protected:
	/**
	 * Struktur zum Halten von Punkt-Informationen. Könnte noch um Textur o.ä.
	 * erweitert werden
	 */
	struct Data {
		GLfloat vertex[3];
		GLfloat normal[3];
	};

	/**
	 * Positionsvektor
	 */
	Vector* position;

	/**
	 * Vektor der angestrebten Position
	 */
	Vector* destination;

	/**
	 * Letzte Position: Wird verwendet um die Drehung zu berechnen
	 */
	Vector* lastPosition;

	/**
	 * Letzter Drehwinkel: Wird verwendet, um Zappeln zu vermeiden
	 */
	int lastAngle;

	/**
	 * Aktueller Drehwinkel
	 */
	int angle;

	/**
	 * Liste aller Fische, für Schwarmverhalten
	 */
	static list<Fish*> fishList;

	/**
	 * Soll der Fisch anderen Fischen gegenüber als "freundlich" auftreten?
	 */
	bool friendly;

	/**
	 * Der Schwanz muss behalten werden, damit er von der Animation gelöscht werden kann
	 */
	Transform* tail;

	/**
	 * Die Flosse muss behalten werden, damit sie von der Animation gelöscht werden kann
	 */
	Transform* leftFin;

	/**
	 * Die Flosse muss behalten werden, damit sie von der Animation gelöscht werden kann
	 */
	Transform* rightFin;

	/**
	 * Children-Node für Flossen und Augen
	 */
	Children* fishParts;

	/**
	 * Zählt die Anzahl der bösen Fische mit
	 */
	static int unfriendlies;

	/**
	 * Vertex-Daten
	 */
	static struct Data data[35];

	/**
	 * Wird einmalig aufgerufen um die Torso-unabhängigen Teile zu erzeugen
	 */
	void constructParts();

	/**
	 * Bewegt den Fisch
	 */
	virtual bool move();

public:
	/**
	 * Statische Methode, die alle Fische in "Schwarmverhalten" bewegt
	 */
	static bool moveAll();

	/**
	 * Liefert den ersten Fisch
	 */
	static Fish* getFirst();

	/**
	 * Liefert den Positionsvektor des Fisches
	 */
	Vector* getPosition();

	/**
	 * Konstruktor: Fisch mit Material und Name
	 */
	Fish(Material* mat, Text* name, Vector* position, Vector* direction);

	/**
	 * Liefert die friendly-Eigenschaft des Fisches
	 */
	bool getFriendly();

	/**
	 * Zeichnet das Objekt
	 */
	virtual void draw();

	/**
	 * Default-Destruktor
	 */
	virtual ~Fish();
};

#endif /*FISH_H_*/
