/* Material.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef MATERIAL_H_
#define MATERIAL_H_
#include <iostream>
#include <cstdlib>
#include <GL/glut.h>
#include "Textures.h"
#include "Vector.h"
using std::cout;
#define material new Material

/**
 * Repräsentiert ein Material. Dieses wird von Objekten aggregiert und hat eine
 * Methode select(). In einer Klassenvariable wird sich das selektierte Material
 * gemerkt und das nächste Material nur selektiert, wenn es nicht das selbe ist.
 */
class Material {
private:
	/**
	 * Speichert das Standard-Material: Schwarz/Matt
	 */
	static Material* defaultMat;

	/**
	 * Speichert das Fisch-Material
	 */
	static Material* fishMat;

	/**
	 * Speichert das zuletzte selektierte Material, um unnötige Aufrufe zu vermeiden
	 */
	static Material* lastUsed;

	/**
	 * Speichert, ob Materialien verwendet werden sollen
	 */
	static bool enabled;

	/**
	 * Die diffuse Farbe des Materials
	 */
	Color* diffuseColor;

	/**
	 * Die ambient Farbe des Materials
	 */
	Color* ambientColor;

	/**
	 * Die aussendende Farbe des Materials
	 */
	Color* emissiveColor;

	/**
	 * Die spekulare Farbe des Materials
	 */
	Color* specularColor;

	/**
	 * Der "shininess"-Wert
	 */
	double shininess;

	/**
	 * Nummer der Textur. -1 für keine Textur
	 */
	int texture;

	/**
	 * Methode, die das Standard-Material erzeugt
	 */
	static Material* createDefaultMaterial();

	/**
	 * Methode, die das Fisch-Material erzeugt
	 */
	static Material* createFishMaterial();

public:
	/**
	 * Default-Konstruktor. Aufgrund der Vielzahl an Kombinationsmöglichkeiten von
	 * möglichen Parametern, kann man die einzelnen Werte nur über Setter-Methoden setzen.
	 */
	Material();

	/**
	 * Liefert das Standard-Material
	 */
	static Material* getDefaultMaterial();

	/**
	 * Liefert das Fisch-Material
	 */
	static Material* getFishMaterial();

	/**
	 * Sollen Materialen selektiert werden können?
	 */
	static void setEnabled(bool enabled);

	/**
	 * Liefert den ambient-Farbwert
	 */
	Color* getAmbientColor();

	/**
	 * Liefert den diffuse-Farbwert
	 */
	Color* getDiffuseColor();

	/**
	 * Liefert den emissive-Farbwert
	 */
	Color* getEmissiveColor();

	/**
	 * Setzt die diffuse Farbe
	 */
	virtual void setDiffuseColor(Color* c);

	/**
	 * Setzt die ambient Farbe
	 */
	virtual void setAmbientColor(Color* c);

	/**
	 * Setzt die aussendende Farbe
	 */
	virtual void setEmissiveColor(Color* c);

	/**
	 * Setzt die spekulare Farbe
	 */
	virtual void setspecularColor(Color* c);

	/**
	 * Setzt den "shininess"-Wert
	 */
	virtual void setShininess(double d);

	/**
	 * Setzt die Textur. -1 für keine Textur
	 */
	virtual void setTexture(int t);

	/**
	 * Aktiviert das Material
	 */
	virtual void select();

	/**
	 * Default-Destruktor
	 */
	virtual ~Material();
};

#endif /*MATERIAL_H_*/
