/* Scene.h
 *
 * fishtank - fish swarm simulation
 * by Andreas Textor <a.textor@kantico.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SCENE_H_
#define SCENE_H_
#include <iostream>
#include <sstream>
#include <string>
#include <list>
#include <iomanip>
#include <cstdlib>
#include <GL/glut.h>
#include "Object.h"
#include "Translation.h"
#include "Rotation.h"
#include "Scale.h"
#include "Children.h"
#include "Material.h"
#include "Vector.h"
#include "Animation.h"
#include "Text.h"
#include "Fish.h"
#include "Shark.h"
using std::cout;
using std::cerr;
using std::endl;
using std::string;
using std::list;
using std::ostringstream;
using std::setw;
using std::setfill;

/**
 * Objekt, das die Szene darstellt. Initialisiert die Grafikausgabe, registriert
 * Callback-Funktionen und zeichnet regelmässig den Szenengraphen
 */
class Scene {
public:
	static Object* namedObject;

	/**
	 * Init-Methode zum Starten des Graphik-Systems
	 */
	static Scene* init(int width, int height, string title, bool fullscreen = false);

	/**
	 * Neuen Knoten zum Szenengraphen zufügen
	 */
	void addNode(Node* nde);

	/**
	 * Neuen Fisch zum Szenengraphen zufügen
	 */
	void addFish(Fish* nde);

	/**
	 * Neuen Hai zum Szenengraphen zufügen
	 */
	void addShark(Shark* nde);

	/**
	 * Startet die glutMainLoop
	 */
	virtual void run();

	/**
	 * Default-Destruktor. Falls die Auflösung umgestellt wurde, wird hier die
	 * systemnative Auflösung wieder hergestellt
	 */
	virtual ~Scene();

private:
	/**
	 * Schreibt die Screenshot-Datei mit fortlaufendem Namen
	 */
	static void screenshot();

	/**
	 * Speichert, ob sich die Ausgabe im Vollbild befindet
	 */
	bool fullscreen;

	/**
	 * Speichert, ob die Fischnamen angezeigt werden soll
	 */
	static bool drawNames;

	/**
	 * Speichert, ob die Onlinehilfe angezeigt werden soll
	 */
	static bool drawHelp;

	/**
	 * Speichert, ob Screenshots geschrieben werden
	 */
	static bool tgaWriter;

	/**
	 * Gibt an, ob die maximale Anzahl von Fischen berechnet wird
	 */
	static bool calculateFishCount;

	/**
	 * Gibt an, ob jegliche Bewegung pausiert ist
	 */
	static bool paused;

	/**
	 * Maximale Anzahl von Fischen, bevor die FPS unter 10 fallen
	 */
	static int maxFishCount;

	/**
	 * Textobjekt zur Anzeige von FPS
	 */
	static Text* fpsDisplay;

	/**
	 * Textobjekt zur Anzeige von Onlinehilfe
	 */
	static list<Text*> helpDisplay;

	/**
	 * Zählt die Frames
	 */
	static int frames;

	/**
	 * Speichert die Nummer der aktiven Kamera
	 */
	static int activeCam;

	/**
	 * Dämpfungswert für leuchtenden Fisch
	 */
	static float attenuation;

	/**
	 * Ist die Deckenlampe an?
	 */
	static bool topLight;

	/**
	 * Speichert statische Kamerapositionen
	 */
	static Vector* camPos[4];

	/**
	 * Scene ist als Singleton implementiert, weil u.A. Callback-Funktionen keine
	 * Exemplarmethoden sein können, wohl aber Klassenmethoden. Hier wird die Instanz
	 * abgespeichert
	 */
	static Scene* singleton;

	/**
	 * Speichert den Szenengraph als Children-Node, der beliebig viele Kindknoten haben kann
	 */
	static Children* graph;

	/**
	 * Speichert die Liste der Fische, die zusätzlich im Szenengraph hängen sollen
	 */
	static Children* fishList;

	/**
	 * Speichert die Liste der Haie, die zusätzlich im Szenengraph hängen sollen
	 */
	static Children* sharkList;

	/**
	 * Konstruktor, der das Singleton-Objekt anlegt
	 */
	Scene(int width, int height, string title, bool fullscreen = false);

	/**
	 * Wählt die nächste verfügbare Kamera aus
	 */
	static void nextCam();

	/**
	 * Callback-Methode für glutTimerFunc
	 */
	static void timer(int id);

	/**
	 * Callback-Methode zum FPS zählen
	 */
	static void fpsTimer(int id);

	/**
	 * Callback-Methode für glutDisplayFunc
	 */
	static void display();

	/**
	 * Callback-Methode für glutReshapeFunc
	 */
	static void reshapeHandler(int w, int h);

	/**
	 * Callback-Methode für glutKeyboardFunc
	 */
	static void keyboardDownHandler(unsigned char key, int x, int y);

	/**
	 * Callback-Methode für glutKeyboardUpFunc
	 */
	static void keyboardUpHandler(unsigned char key, int x, int y);

	/**
	 * Callback-Methode für glutMouseFunc
	 */
	static void mouseHandler(int button, int state, int u, int v);
};

#endif /*SCENE_H_*/
